@@ -2,6 +2,8 @@
 # Created by Pearu Peterson, September 2002
 """ Test functions for fftpack.basic module
 """
+from __future__ import division, print_function, absolute_import
+
 __usage__ = """
 Build fftpack:
   python setup_fftpack.py build
@@ -11,7 +13,9 @@ Run tests if fftpack is not installed:
   python tests/test_basic.py
 """
 
-from numpy.testing import *
+from numpy.testing import assert_, assert_equal, assert_array_almost_equal, \
+        assert_array_almost_equal_nulp, assert_raises, run_module_suite, \
+        TestCase, dec
 from scipy.fftpack import ifft,fft,fftn,ifftn,rfft,irfft, fft2
 from scipy.fftpack import _fftpack as fftpack
 
@@ -83,7 +87,7 @@ def direct_rdft(x):
     n = len(x)
     w = -arange(n)*(2j*pi/n)
     r = zeros(n,dtype=double)
-    for i in range(int(n/2+1)):
+    for i in range(n//2+1):
         y = dot(exp(i*w),x)
         if i:
             r[2*i-1] = y.real
@@ -97,7 +101,7 @@ def direct_irdft(x):
     x = asarray(x)
     n = len(x)
     x1 = zeros(n,dtype=cdouble)
-    for i in range(int(n/2+1)):
+    for i in range(n//2+1):
         if i:
             if 2*i<n:
                 x1[i] = x[2*i-1] + 1j* x[2*i]
@@ -112,11 +116,12 @@ class _TestFFTBase(TestCase):
     def setUp(self):
         self.cdt = None
         self.rdt = None
+        np.random.seed(1234)
 
     def test_definition(self):
         x = np.array([1,2,3,4+1j,1,2,3,4+2j], dtype = self.cdt)
         y = fft(x)
-        self.failUnless(y.dtype == self.cdt,
+        self.assertTrue(y.dtype == self.cdt,
                 "Output dtype is %s, expected %s" % (y.dtype, self.cdt))
         y1 = direct_dft(x)
         assert_array_almost_equal(y,y1)
@@ -127,7 +132,7 @@ class _TestFFTBase(TestCase):
         x1 = np.array([1,2,3,4], dtype=self.rdt)
         x2 = np.array([1,2,3,4], dtype=self.rdt)
         y = fft([x1,x2],n=4)
-        self.failUnless(y.dtype == self.cdt,
+        self.assertTrue(y.dtype == self.cdt,
                 "Output dtype is %s, expected %s" % (y.dtype, self.cdt))
         assert_equal(y.shape,(2,4))
         assert_array_almost_equal(y[0],direct_dft(x1))
@@ -137,7 +142,7 @@ class _TestFFTBase(TestCase):
         x1 = np.array([1,2,3,4+1j], dtype=self.cdt)
         x2 = np.array([1,2,3,4+1j], dtype=self.cdt)
         y = fft([x1,x2],n=4)
-        self.failUnless(y.dtype == self.cdt,
+        self.assertTrue(y.dtype == self.cdt,
                 "Output dtype is %s, expected %s" % (y.dtype, self.cdt))
         assert_equal(y.shape,(2,4))
         assert_array_almost_equal(y[0],direct_dft(x1))
@@ -146,7 +151,7 @@ class _TestFFTBase(TestCase):
     def test_djbfft(self):
         for i in range(2,14):
             n = 2**i
-            x = range(n)
+            x = list(range(n))
             y = fftpack.zfft(x)
             y2 = numpy.fft.fft(x)
             assert_array_almost_equal(y,y2)
@@ -168,11 +173,14 @@ class TestSingleFFT(_TestFFTBase):
         pass
 
 class _TestIFFTBase(TestCase):
+    def setUp(self):
+        np.random.seed(1234)
+
     def test_definition(self):
         x = np.array([1,2,3,4+1j,1,2,3,4+2j], self.cdt)
         y = ifft(x)
         y1 = direct_idft(x)
-        self.failUnless(y.dtype == self.cdt,
+        self.assertTrue(y.dtype == self.cdt,
                 "Output dtype is %s, expected %s" % (y.dtype, self.cdt))
         assert_array_almost_equal(y,y1)
 
@@ -182,20 +190,20 @@ class _TestIFFTBase(TestCase):
     def test_definition_real(self):
         x = np.array([1,2,3,4,1,2,3,4], self.rdt)
         y = ifft(x)
-        self.failUnless(y.dtype == self.cdt,
+        self.assertTrue(y.dtype == self.cdt,
                 "Output dtype is %s, expected %s" % (y.dtype, self.cdt))
         y1 = direct_idft(x)
         assert_array_almost_equal(y,y1)
 
         x = np.array([1,2,3,4,5], dtype=self.rdt)
-        self.failUnless(y.dtype == self.cdt,
+        self.assertTrue(y.dtype == self.cdt,
                 "Output dtype is %s, expected %s" % (y.dtype, self.cdt))
         assert_array_almost_equal(ifft(x),direct_idft(x))
 
     def test_djbfft(self):
         for i in range(2,14):
             n = 2**i
-            x = range(n)
+            x = list(range(n))
             y = fftpack.zfft(x,direction=-1)
             y2 = numpy.fft.ifft(x)
             assert_array_almost_equal(y,y2)
@@ -208,9 +216,9 @@ class _TestIFFTBase(TestCase):
             x = random([size]).astype(self.cdt) +1j*x
             y1 = ifft(fft(x))
             y2 = fft(ifft(x))
-            self.failUnless(y1.dtype == self.cdt,
+            self.assertTrue(y1.dtype == self.cdt,
                     "Output dtype is %s, expected %s" % (y1.dtype, self.cdt))
-            self.failUnless(y2.dtype == self.cdt,
+            self.assertTrue(y2.dtype == self.cdt,
                     "Output dtype is %s, expected %s" % (y2.dtype, self.cdt))
             assert_array_almost_equal (y1, x)
             assert_array_almost_equal (y2, x)
@@ -220,9 +228,9 @@ class _TestIFFTBase(TestCase):
             x = random([size]).astype(self.rdt)
             y1 = ifft(fft(x))
             y2 = fft(ifft(x))
-            self.failUnless(y1.dtype == self.cdt,
+            self.assertTrue(y1.dtype == self.cdt,
                     "Output dtype is %s, expected %s" % (y1.dtype, self.cdt))
-            self.failUnless(y2.dtype == self.cdt,
+            self.assertTrue(y2.dtype == self.cdt,
                     "Output dtype is %s, expected %s" % (y2.dtype, self.cdt))
             assert_array_almost_equal (y1, x)
             assert_array_almost_equal (y2, x)
@@ -238,18 +246,18 @@ class _TestIFFTBase(TestCase):
             np.random.seed(1234)
             x = np.random.rand(size).astype(self.rdt)
             y = ifft(fft(x))
-            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+            self.assertTrue(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
                             (size, self.rdt))
             y = fft(ifft(x))
-            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+            self.assertTrue(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
                             (size, self.rdt))
 
             x = (x + 1j*np.random.rand(size)).astype(self.cdt)
             y = ifft(fft(x))
-            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+            self.assertTrue(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
                             (size, self.rdt))
             y = fft(ifft(x))
-            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+            self.assertTrue(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
                             (size, self.rdt))
 
 class TestDoubleIFFT(_TestIFFTBase):
@@ -263,6 +271,8 @@ class TestSingleIFFT(_TestIFFTBase):
         self.rdt = np.float32
 
 class _TestRFFTBase(TestCase):
+    def setUp(self):
+        np.random.seed(1234)
 
     def test_definition(self):
         for t in [[1, 2, 3, 4, 1, 2, 3, 4], [1, 2, 3, 4, 1, 2, 3, 4, 5]]:
@@ -270,19 +280,19 @@ class _TestRFFTBase(TestCase):
             y = rfft(x)
             y1 = direct_rdft(x)
             assert_array_almost_equal(y,y1)
-            self.failUnless(y.dtype == self.rdt,
+            self.assertTrue(y.dtype == self.rdt,
                     "Output dtype is %s, expected %s" % (y.dtype, self.rdt))
 
     def test_djbfft(self):
         from numpy.fft import fft as numpy_fft
         for i in range(2,14):
             n = 2**i
-            x = range(n)
+            x = list(range(n))
             y2 = numpy_fft(x)
             y1 = zeros((n,),dtype=double)
             y1[0] = y2[0].real
-            y1[-1] = y2[n/2].real
-            for k in range(1, int(n/2)):
+            y1[-1] = y2[n//2].real
+            for k in range(1, n//2):
                 y1[2*k-1] = y2[k].real
                 y1[2*k] = y2[k].imag
             y = fftpack.drfft(x)
@@ -299,6 +309,8 @@ class TestRFFTSingle(_TestRFFTBase):
         self.rdt = np.float32
 
 class _TestIRFFTBase(TestCase):
+    def setUp(self):
+        np.random.seed(1234)
 
     def test_definition(self):
         x1 = [1,2,3,4,1,2,3,4]
@@ -309,7 +321,7 @@ class _TestIRFFTBase(TestCase):
         def _test(x, xr):
             y = irfft(np.array(x, dtype=self.rdt))
             y1 = direct_irdft(x)
-            self.failUnless(y.dtype == self.rdt,
+            self.assertTrue(y.dtype == self.rdt,
                     "Output dtype is %s, expected %s" % (y.dtype, self.rdt))
             assert_array_almost_equal(y,y1, decimal=self.ndec)
             assert_array_almost_equal(y,ifft(xr), decimal=self.ndec)
@@ -321,13 +333,13 @@ class _TestIRFFTBase(TestCase):
         from numpy.fft import ifft as numpy_ifft
         for i in range(2,14):
             n = 2**i
-            x = range(n)
+            x = list(range(n))
             x1 = zeros((n,),dtype=cdouble)
             x1[0] = x[0]
-            for k in range(1, int(n/2)):
+            for k in range(1, n//2):
                 x1[k] = x[2*k-1]+1j*x[2*k]
                 x1[n-k] = x[2*k-1]-1j*x[2*k]
-            x1[n/2] = x[-1]
+            x1[n//2] = x[-1]
             y1 = numpy_ifft(x1)
             y = fftpack.drfft(x,direction=-1)
             assert_array_almost_equal(y,y1)
@@ -337,9 +349,9 @@ class _TestIRFFTBase(TestCase):
             x = random([size]).astype(self.rdt)
             y1 = irfft(rfft(x))
             y2 = rfft(irfft(x))
-            self.failUnless(y1.dtype == self.rdt,
+            self.assertTrue(y1.dtype == self.rdt,
                     "Output dtype is %s, expected %s" % (y1.dtype, self.rdt))
-            self.failUnless(y2.dtype == self.rdt,
+            self.assertTrue(y2.dtype == self.rdt,
                     "Output dtype is %s, expected %s" % (y2.dtype, self.rdt))
             assert_array_almost_equal (y1, x, decimal=self.ndec,
                                        err_msg="size=%d" % size)
@@ -357,10 +369,10 @@ class _TestIRFFTBase(TestCase):
             np.random.seed(1234)
             x = np.random.rand(size).astype(self.rdt)
             y = irfft(rfft(x))
-            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+            self.assertTrue(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
                             (size, self.rdt))
             y = rfft(irfft(x))
-            self.failUnless(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
+            self.assertTrue(np.linalg.norm(x - y) < rtol*np.linalg.norm(x),
                             (size, self.rdt))
 
 # self.ndec is bogus; we should have a assert_array_approx_equal for number of
@@ -378,6 +390,9 @@ class TestIRFFTSingle(_TestIRFFTBase):
         self.ndec = 5
 
 class Testfft2(TestCase):
+    def setUp(self):
+        np.random.seed(1234)
+
     def test_regression_244(self):
         """fft returns wrong result with axes parameter."""
         # fftn (and hence fft2) used to break when both axes and shape were
@@ -388,6 +403,9 @@ class Testfft2(TestCase):
         assert_array_almost_equal(y, y_r)
 
 class TestFftnSingle(TestCase):
+    def setUp(self):
+        np.random.seed(1234)
+
     def test_definition(self):
         x = [[1,2,3],[4,5,6],[7,8,9]]
         y = fftn(np.array(x, np.float32))
@@ -401,22 +419,24 @@ class TestFftnSingle(TestCase):
         for size in SMALL_COMPOSITE_SIZES + SMALL_PRIME_SIZES:
             np.random.seed(1234)
             x = np.random.rand(size, size) + 1j*np.random.rand(size, size)
-            y1 = fftn(x.astype(np.float32))
-            y2 = fftn(x.astype(np.float64)).astype(np.complex64)
+            y1 = fftn(x.real.astype(np.float32))
+            y2 = fftn(x.real.astype(np.float64)).astype(np.complex64)
 
-            self.failUnless(y1.dtype == np.complex64)
+            self.assertTrue(y1.dtype == np.complex64)
             assert_array_almost_equal_nulp(y1, y2, 2000)
 
         for size in LARGE_COMPOSITE_SIZES + LARGE_PRIME_SIZES:
             np.random.seed(1234)
             x = np.random.rand(size, 3) + 1j*np.random.rand(size, 3)
-            y1 = fftn(x.astype(np.float32))
-            y2 = fftn(x.astype(np.float64)).astype(np.complex64)
+            y1 = fftn(x.real.astype(np.float32))
+            y2 = fftn(x.real.astype(np.float64)).astype(np.complex64)
 
-            self.failUnless(y1.dtype == np.complex64)
+            self.assertTrue(y1.dtype == np.complex64)
             assert_array_almost_equal_nulp(y1, y2, 2000)
 
 class TestFftn(TestCase):
+    def setUp(self):
+        np.random.seed(1234)
 
     def test_definition(self):
         x = [[1,2,3],[4,5,6],[7,8,9]]
@@ -561,23 +581,22 @@ class TestFftn(TestCase):
         assert_array_almost_equal(y, numpy.fft.fftn(x, axes=(-3, -2), s=(8, 8)))
 
     def test_shape_argument_more(self):
-        # Test that fftn raise a value error exception when s.shape is longer
-        # than x.shape
+        """Test that fftn raises ValueError when s.shape is longer than x.shape"""
         x = zeros((4, 4, 2))
-        try:
-            fx = fftn(x, shape = (8, 8, 2, 1))
-            raise AssertionError("s.shape longer than x.shape succeded, "\
-                                 "but should not have.")
-        except ValueError:
-            pass
+        assert_raises(ValueError, fftn, x, shape=(8, 8, 2, 1))
+
 
 class _TestIfftn(TestCase):
     dtype = None
     cdtype = None
+
+    def setUp(self):
+        np.random.seed(1234)
+
     def test_definition(self):
         x = np.array([[1,2,3],[4,5,6],[7,8,9]], dtype=self.dtype)
         y = ifftn(x)
-        assert y.dtype == self.cdtype
+        assert_(y.dtype == self.cdtype)
         assert_array_almost_equal_nulp(y,direct_idftn(x),self.maxnlp)
         x = random((20,26))
         assert_array_almost_equal_nulp(ifftn(x),direct_idftn(x),self.maxnlp)
@@ -601,6 +620,8 @@ class TestIfftnSingle(_TestIfftn):
     maxnlp = 3500
 
 class TestLongDoubleFailure(TestCase):
+    def setUp(self):
+        np.random.seed(1234)
 
     def test_complex(self):
         if np.dtype(np.longcomplex).itemsize == np.dtype(np.complex).itemsize:
@@ -633,5 +654,149 @@ class TestLongDoubleFailure(TestCase):
             except ValueError:
                 pass
 
+
+
+class FakeArray(object):
+    def __init__(self, data):
+        self._data = data
+        self.__array_interface__ = data.__array_interface__
+
+class FakeArray2(object):
+    def __init__(self, data):
+        self._data = data
+    def __array__(self):
+        return self._data
+
+class TestOverwrite(object):
+    """
+    Check input overwrite behavior of the FFT functions
+    """
+
+    real_dtypes = [np.float32, np.float64]
+    dtypes = real_dtypes + [np.complex64, np.complex128]
+
+    def _check(self, x, routine, fftsize, axis, overwrite_x, should_overwrite):
+        x2 = x.copy()
+        for fake in [lambda x: x, FakeArray, FakeArray2]:
+            y = routine(fake(x2), fftsize, axis, overwrite_x=overwrite_x)
+
+            sig = "%s(%s%r, %r, axis=%r, overwrite_x=%r)" % (
+                routine.__name__, x.dtype, x.shape, fftsize, axis, overwrite_x)
+            if not should_overwrite:
+                assert_equal(x2, x, err_msg="spurious overwrite in %s" % sig)
+            else:
+                if (x2 == x).all():
+                    raise AssertionError("no overwrite in %s" % sig)
+
+    def _check_1d(self, routine, dtype, shape, axis, overwritable_dtypes):
+        np.random.seed(1234)
+        if np.issubdtype(dtype, np.complexfloating):
+            data = np.random.randn(*shape) + 1j*np.random.randn(*shape)
+        else:
+            data = np.random.randn(*shape)
+        data = data.astype(dtype)
+
+        for fftsize in [8, 16, 32]:
+            for overwrite_x in [True, False]:
+                should_overwrite = (overwrite_x
+                                    and dtype in overwritable_dtypes
+                                    and fftsize <= shape[axis]
+                                    and (len(shape) == 1 or
+                                         (axis % len(shape) == len(shape)-1
+                                          and fftsize == shape[axis])))
+                self._check(data, routine, fftsize, axis,
+                            overwrite_x=overwrite_x,
+                            should_overwrite=should_overwrite)
+
+    def test_fft(self):
+        overwritable = (np.complex128, np.complex64)
+        for dtype in self.dtypes:
+            self._check_1d(fft, dtype, (16,), -1, overwritable)
+            self._check_1d(fft, dtype, (16, 2), 0, overwritable)
+            self._check_1d(fft, dtype, (2, 16), 1, overwritable)
+
+    def test_ifft(self):
+        overwritable = (np.complex128, np.complex64)
+        for dtype in self.dtypes:
+            self._check_1d(ifft, dtype, (16,), -1, overwritable)
+            self._check_1d(ifft, dtype, (16, 2), 0, overwritable)
+            self._check_1d(ifft, dtype, (2, 16), 1, overwritable)
+
+    def test_rfft(self):
+        overwritable = self.real_dtypes
+        for dtype in self.real_dtypes:
+            self._check_1d(rfft, dtype, (16,), -1, overwritable)
+            self._check_1d(rfft, dtype, (16, 2), 0, overwritable)
+            self._check_1d(rfft, dtype, (2, 16), 1, overwritable)
+
+    def test_irfft(self):
+        overwritable = self.real_dtypes
+        for dtype in self.real_dtypes:
+            self._check_1d(irfft, dtype, (16,), -1, overwritable)
+            self._check_1d(irfft, dtype, (16, 2), 0, overwritable)
+            self._check_1d(irfft, dtype, (2, 16), 1, overwritable)
+
+    def _check_nd_one(self, routine, dtype, shape, axes, overwritable_dtypes):
+        np.random.seed(1234)
+        if np.issubdtype(dtype, np.complexfloating):
+            data = np.random.randn(*shape) + 1j*np.random.randn(*shape)
+        else:
+            data = np.random.randn(*shape)
+        data = data.astype(dtype)
+
+        def fftshape_iter(shp):
+            if len(shp) <= 0:
+                yield ()
+            else:
+                for j in (shp[0]//2, shp[0], shp[0]*2):
+                    for rest in fftshape_iter(shp[1:]):
+                        yield (j,) + rest
+
+        if axes is None:
+            part_shape = shape
+        else:
+            part_shape = tuple(np.take(shape, axes))
+
+        for overwrite_x in [True, False]:
+            for fftshape in fftshape_iter(part_shape):
+                should_overwrite = (overwrite_x
+                                    and data.ndim == 1
+                                    and np.all([x < y for x, y in zip(fftshape, part_shape)])
+                                    and dtype in overwritable_dtypes)
+                self._check(data, routine, fftshape, axes,
+                            overwrite_x=overwrite_x,
+                            should_overwrite=should_overwrite)
+                if data.ndim > 1:
+                    # check fortran order: it never overwrites
+                    self._check(data.T, routine, fftshape, axes,
+                                overwrite_x=overwrite_x,
+                                should_overwrite=False)
+
+    def _check_nd(self, routine, dtype, overwritable):
+        self._check_nd_one(routine, dtype, (16,), None, overwritable)
+        self._check_nd_one(routine, dtype, (16,), (0,), overwritable)
+        self._check_nd_one(routine, dtype, (16, 2), (0,), overwritable)
+        self._check_nd_one(routine, dtype, (2, 16), (1,), overwritable)
+        self._check_nd_one(routine, dtype, (8, 16), None, overwritable)
+        self._check_nd_one(routine, dtype, (8, 16), (0, 1), overwritable)
+        self._check_nd_one(routine, dtype, (8, 16, 2), (0, 1), overwritable)
+        self._check_nd_one(routine, dtype, (8, 16, 2), (1, 2), overwritable)
+        self._check_nd_one(routine, dtype, (8, 16, 2), (0,), overwritable)
+        self._check_nd_one(routine, dtype, (8, 16, 2), (1,), overwritable)
+        self._check_nd_one(routine, dtype, (8, 16, 2), (2,), overwritable)
+        self._check_nd_one(routine, dtype, (8, 16, 2), None, overwritable)
+        self._check_nd_one(routine, dtype, (8, 16, 2), (0,1,2), overwritable)
+
+    def test_fftn(self):
+        overwritable = (np.complex128, np.complex64)
+        for dtype in self.dtypes:
+            self._check_nd(fftn, dtype, overwritable)
+
+    def test_ifftn(self):
+        overwritable = (np.complex128, np.complex64)
+        for dtype in self.dtypes:
+            self._check_nd(ifftn, dtype, overwritable)
+
+
 if __name__ == "__main__":
     run_module_suite()
