@@ -1,4 +1,5 @@
 ''' Tests for netcdf '''
+from __future__ import division, print_function, absolute_import
 
 import os
 from os.path import join as pjoin, dirname
@@ -6,15 +7,11 @@ import shutil
 import tempfile
 import time
 import sys
-if sys.version_info[0] >= 3:
-    from io import BytesIO
-else:
-    from StringIO import StringIO as BytesIO
+from io import BytesIO
 from glob import glob
 
 import numpy as np
-from numpy.compat import asbytes
-from numpy.testing import dec, assert_
+from numpy.testing import dec, assert_, assert_allclose
 
 from scipy.io.netcdf import netcdf_file
 
@@ -39,9 +36,9 @@ def make_simple(*args, **kwargs):
 
 def gen_for_simple(ncfileobj):
     ''' Generator for example fileobj tests '''
-    yield assert_equal, ncfileobj.history, asbytes('Created for a test')
+    yield assert_equal, ncfileobj.history, b'Created for a test'
     time = ncfileobj.variables['time']
-    yield assert_equal, time.units, asbytes('days since 2008-01-01')
+    yield assert_equal, time.units, b'days since 2008-01-01'
     yield assert_equal, time.shape, (N_EG_ELS,)
     yield assert_equal, time[-1], N_EG_ELS-1
 
@@ -178,3 +175,27 @@ def test_dtype_specifiers():
     v3 = f.createVariable('v3', np.dtype(np.int16), ['x'])
     f.close()
 
+
+def test_ticket_1720():
+    io = BytesIO()
+
+    items = [0,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9]
+
+    f = netcdf_file(io, 'w')
+    f.history = 'Created for a test'
+    f.createDimension('float_var', 10)
+    float_var = f.createVariable('float_var', 'f', ('float_var',))
+    float_var[:] = items
+    float_var.units = 'metres'
+    f.flush()
+    contents = io.getvalue()
+    f.close()
+
+    io = BytesIO(contents)
+    f = netcdf_file(io, 'r')
+    assert_equal(f.history, b'Created for a test')
+    float_var = f.variables['float_var']
+    assert_equal(float_var.units, b'metres')
+    assert_equal(float_var.shape, (10,))
+    assert_allclose(float_var[:], items)
+    f.close()
