@@ -1,98 +1,76 @@
-#this program corresponds to special.py
-from decimal import Decimal
-import types
+from __future__ import division, print_function, absolute_import
 
-from numpy.testing import *
+import warnings
+from decimal import Decimal
+
+from numpy.testing import TestCase, run_module_suite, assert_equal, \
+    assert_almost_equal, assert_array_equal, assert_array_almost_equal, \
+    assert_raises, assert_
 
 import scipy.signal as signal
-from scipy.signal import lfilter, correlate, convolve, convolve2d, hilbert
+from scipy.signal import correlate, convolve, convolve2d, fftconvolve, \
+     hilbert, hilbert2, lfilter, lfilter_zi, filtfilt, butter, tf2zpk
 
 
 from numpy import array, arange
 import numpy as np
 
+
 class _TestConvolve(TestCase):
     def test_basic(self):
         a = [3,4,5,6,5,4]
         b = [1,2,3]
-        c = convolve(a,b, old_behavior=self.old_behavior)
+        c = convolve(a,b)
         assert_array_equal(c,array([3,10,22,28,32,32,23,12]))
 
     def test_complex(self):
         x = array([1+1j, 2+1j, 3+1j])
         y = array([1+1j, 2+1j])
-        z = convolve(x, y,old_behavior=self.old_behavior)
+        z = convolve(x, y)
         assert_array_equal(z, array([2j, 2+6j, 5+8j, 5+5j]))
 
-    def test_zero_order(self):
+    def test_zero_rank(self):
         a = 1289
         b = 4567
-        c = convolve(a,b,old_behavior=self.old_behavior)
-        assert_array_equal(c,a*b)
+        c = convolve(a,b)
+        assert_equal(c,a*b)
+
+    def test_single_element(self):
+        a = array([4967])
+        b = array([3920])
+        c = convolve(a,b)
+        assert_equal(c,a*b)
 
     def test_2d_arrays(self):
         a = [[1,2,3],[3,4,5]]
         b = [[2,3,4],[4,5,6]]
-        c = convolve(a,b,old_behavior=self.old_behavior)
-        d = array(  [[2 ,7 ,16,17,12],\
-                     [10,30,62,58,38],\
-                     [12,31,58,49,30]])
+        c = convolve(a,b)
+        d = array([[2 ,7 ,16,17,12],
+                   [10,30,62,58,38],
+                   [12,31,58,49,30]])
         assert_array_equal(c,d)
 
     def test_valid_mode(self):
         a = [1,2,3,6,5,3]
         b = [2,3,4,5,3,4,2,2,1]
-        c = convolve(a,b,'valid',old_behavior=self.old_behavior)
+        c = convolve(a,b,'valid')
         assert_array_equal(c,array([70,78,73,65]))
 
-class OldTestConvolve(_TestConvolve):
-    old_behavior = True
-    @dec.deprecated()
-    def test_basic(self):
-        _TestConvolve.test_basic(self)
-
-    @dec.deprecated()
-    def test_complex(self):
-        _TestConvolve.test_complex(self)
-
-    @dec.deprecated()
-    def test_2d_arrays(self):
-        _TestConvolve.test_2d_arrays(self)
-
-    @dec.deprecated()
-    def test_same_mode(self):
-        _TestConvolve.test_same_mode(self)
-
-    @dec.deprecated()
-    def test_valid_mode(self):
-        a = [1,2,3,6,5,3]
-        b = [2,3,4,5,3,4,2,2,1]
-        c = convolve(a,b,'valid',old_behavior=self.old_behavior)
-        assert_array_equal(c,array([70,78,73,65]))
-
-    @dec.deprecated()
-    def test_same_mode(self):
-        a = [1,2,3,3,1,2]
-        b = [1,4,3,4,5,6,7,4,3,2,1,1,3]
-        c = convolve(a,b,'same',old_behavior=self.old_behavior)
-        d = array([14,25,35,43,57,61,63,57,45,36,25,20,17])
-        assert_array_equal(c,d)
 
 class TestConvolve(_TestConvolve):
-    old_behavior = False
     def test_valid_mode(self):
         # 'valid' mode if b.size > a.size does not make sense with the new
         # behavior
         a = [1,2,3,6,5,3]
         b = [2,3,4,5,3,4,2,2,1]
         def _test():
-            convolve(a,b,'valid',old_behavior=self.old_behavior)
-        self.failUnlessRaises(ValueError, _test)
+            convolve(a,b,'valid')
+        self.assertRaises(ValueError, _test)
 
     def test_same_mode(self):
         a = [1,2,3,3,1,2]
         b = [1,4,3,4,5,6,7,4,3,2,1,1,3]
-        c = convolve(a,b,'same',old_behavior=self.old_behavior)
+        c = convolve(a,b,'same')
         d = array([57,61,63,57,45,36])
         assert_array_equal(c,d)
 
@@ -100,106 +78,76 @@ class _TestConvolve2d(TestCase):
     def test_2d_arrays(self):
         a = [[1,2,3],[3,4,5]]
         b = [[2,3,4],[4,5,6]]
-        d = array(  [[2 ,7 ,16,17,12],\
-                     [10,30,62,58,38],\
-                     [12,31,58,49,30]])
-        e = convolve2d(a,b,old_behavior=self.old_behavior)
-        assert_array_equal(e,d)
+        d = array([[2 ,7 ,16,17,12],
+                   [10,30,62,58,38],
+                   [12,31,58,49,30]])
+        e = convolve2d(a, b)
+        assert_array_equal(e, d)
 
     def test_valid_mode(self):
-        e = [[2,3,4,5,6,7,8],[4,5,6,7,8,9,10]]
-        f = [[1,2,3],[3,4,5]]
-        g = convolve2d(e,f,'valid',old_behavior=self.old_behavior)
+        e = [[2,3,4,5,6,7,8], [4,5,6,7,8,9,10]]
+        f = [[1,2,3], [3,4,5]]
+        g = convolve2d(e, f, 'valid')
         h = array([[62,80,98,116,134]])
-        assert_array_equal(g,h)
+        assert_array_equal(g, h)
 
     def test_fillvalue(self):
         a = [[1,2,3],[3,4,5]]
         b = [[2,3,4],[4,5,6]]
         fillval = 1
-        c = convolve2d(a,b,'full','fill',fillval,old_behavior=self.old_behavior)
-        d = array([[24,26,31,34,32],\
-                   [28,40,62,64,52],\
+        c = convolve2d(a,b,'full','fill',fillval)
+        d = array([[24,26,31,34,32],
+                   [28,40,62,64,52],
                    [32,46,67,62,48]])
-        assert_array_equal(c,d)
+        assert_array_equal(c, d)
 
     def test_wrap_boundary(self):
         a = [[1,2,3],[3,4,5]]
         b = [[2,3,4],[4,5,6]]
-        c = convolve2d(a,b,'full','wrap',old_behavior=self.old_behavior)
-        d = array([[80,80,74,80,80],\
-                   [68,68,62,68,68],\
+        c = convolve2d(a,b,'full','wrap')
+        d = array([[80,80,74,80,80],
+                   [68,68,62,68,68],
                    [80,80,74,80,80]])
         assert_array_equal(c,d)
 
     def test_sym_boundary(self):
         a = [[1,2,3],[3,4,5]]
         b = [[2,3,4],[4,5,6]]
-        c = convolve2d(a,b,'full','symm',old_behavior=self.old_behavior)
-        d = array([[34,30,44, 62, 66],\
-                   [52,48,62, 80, 84],\
+        c = convolve2d(a,b,'full','symm')
+        d = array([[34,30,44, 62, 66],
+                   [52,48,62, 80, 84],
                    [82,78,92,110,114]])
         assert_array_equal(c,d)
 
 
-class OldTestConvolve2d(_TestConvolve2d):
-    old_behavior = True
-    @dec.deprecated()
-    def test_2d_arrays(self):
-        _TestConvolve2d.test_2d_arrays(self)
-
-    @dec.deprecated()
+class TestConvolve2d(_TestConvolve2d):
     def test_same_mode(self):
         e = [[1,2,3],[3,4,5]]
         f = [[2,3,4,5,6,7,8],[4,5,6,7,8,9,10]]
-        g = convolve2d(e,f,'same',old_behavior=self.old_behavior)
-        h = array([[ 7,16,22,28, 34, 40, 37],\
-                   [30,62,80,98,116,134,114]])
+        g = convolve2d(e,f,'same')
+        h = array([[22,28,34],
+                   [80,98,116]])
         assert_array_equal(g,h)
 
-    @dec.deprecated()
-    def test_valid_mode(self):
-        _TestConvolve2d.test_valid_mode(self)
-
-    @dec.deprecated()
-    def test_fillvalue(self):
-        _TestConvolve2d.test_fillvalue(self)
-
-    @dec.deprecated()
-    def test_wrap_boundary(self):
-        _TestConvolve2d.test_wrap_boundary(self)
-
-    @dec.deprecated()
-    def test_sym_boundary(self):
-        _TestConvolve2d.test_sym_boundary(self)
-
-    @dec.deprecated()
     def test_valid_mode2(self):
-        # Test when in2.size > in1.size: old behavior is to do so that
-        # convolve2d(in2, in1) == convolve2d(in1, in2)
+        # Test when in2.size > in1.size
         e = [[1,2,3],[3,4,5]]
         f = [[2,3,4,5,6,7,8],[4,5,6,7,8,9,10]]
-        g = convolve2d(e,f,'valid',old_behavior=self.old_behavior)
-        h = array([[62,80,98,116,134]])
-        assert_array_equal(g,h)
+        def _test():
+            convolve2d(e,f,'valid')
+        self.assertRaises(ValueError, _test)
+
+    def test_consistency_convolve_funcs(self):
+        # Compare np.convolve, signal.convolve, signal.convolve2d
+        a = np.arange(5)
+        b = np.array([3.2, 1.4, 3])
+        for mode in ['full', 'valid', 'same']:
+            assert_almost_equal(np.convolve(a, b, mode=mode),
+                                signal.convolve(a, b, mode=mode))
+            assert_almost_equal(np.squeeze(signal.convolve2d([a], [b],
+                                           mode=mode)),
+                                signal.convolve(a, b, mode=mode))
 
-#class TestConvolve2d(_TestConvolve2d):
-#    old_behavior = False
-#    def test_same_mode(self):
-#        e = [[1,2,3],[3,4,5]]
-#        f = [[2,3,4,5,6,7,8],[4,5,6,7,8,9,10]]
-#        g = convolve2d(e,f,'same',old_behavior=self.old_behavior)
-#        h = array([[80,98,116],\
-#                   [70,82,94]])
-#        assert_array_equal(g,h)
-#
-#    def test_valid_mode2(self):
-#        # Test when in2.size > in1.size
-#        e = [[1,2,3],[3,4,5]]
-#        f = [[2,3,4,5,6,7,8],[4,5,6,7,8,9,10]]
-#        def _test():
-#            convolve2d(e,f,'valid',old_behavior=self.old_behavior)
-#        self.failUnlessRaises(ValueError, _test)
 
 class TestFFTConvolve(TestCase):
     def test_real(self):
@@ -213,47 +161,74 @@ class TestFFTConvolve(TestCase):
 
     def test_2d_real_same(self):
         a = array([[1,2,3],[4,5,6]])
-        assert_array_almost_equal(signal.fftconvolve(a,a),\
-                                                array([[1,4,10,12,9],\
-                                                       [8,26,56,54,36],\
-                                                       [16,40,73,60,36]]))
+        assert_array_almost_equal(signal.fftconvolve(a,a),
+                                  array([[1,4,10,12,9],
+                                         [8,26,56,54,36],
+                                         [16,40,73,60,36]]))
 
     def test_2d_complex_same(self):
         a = array([[1+2j,3+4j,5+6j],[2+1j,4+3j,6+5j]])
-        c = signal.fftconvolve(a,a)
-        d = array([[-3+4j,-10+20j,-21+56j,-18+76j,-11+60j],\
-                   [10j,44j,118j,156j,122j],\
+        c = fftconvolve(a,a)
+        d = array([[-3+4j,-10+20j,-21+56j,-18+76j,-11+60j],
+                   [10j,44j,118j,156j,122j],
                    [3+4j,10+20j,21+56j,18+76j,11+60j]])
         assert_array_almost_equal(c,d)
 
     def test_real_same_mode(self):
         a = array([1,2,3])
         b = array([3,3,5,6,8,7,9,0,1])
-        c = signal.fftconvolve(a,b,'same')
+        c = fftconvolve(a,b,'same')
+        d = array([ 35.,  41.,  47.])
+        assert_array_almost_equal(c,d)
+
+    def test_real_same_mode2(self):
+        a = array([3,3,5,6,8,7,9,0,1])
+        b = array([1,2,3])
+        c = fftconvolve(a,b,'same')
         d = array([9.,20.,25.,35.,41.,47.,39.,28.,2.])
         assert_array_almost_equal(c,d)
 
     def test_real_valid_mode(self):
+        # len(a) < len(b) deprecated in 0.12.0, removed for 0.13.0
         a = array([3,2,1])
         b = array([3,3,5,6,8,7,9,0,1])
-        c = signal.fftconvolve(a,b,'valid')
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", DeprecationWarning)
+            assert_array_almost_equal(fftconvolve(a, b, 'valid'),
+                                      fftconvolve(b, a, 'valid'))
+
+    def test_real_valid_mode2(self):
+        a = array([3,3,5,6,8,7,9,0,1])
+        b = array([3,2,1])
+        c = fftconvolve(a,b,'valid')
         d = array([24.,31.,41.,43.,49.,25.,12.])
         assert_array_almost_equal(c,d)
 
-    def test_zero_order(self):
+    def test_empty(self):
+        # Regression test for #1745: crashes with 0-length input.
+        assert_(fftconvolve([], []).size == 0)
+        assert_(fftconvolve([5, 6], []).size == 0)
+        assert_(fftconvolve([], [7]).size == 0)
+
+    def test_zero_rank(self):
+        a = array(4967)
+        b = array(3920)
+        c = fftconvolve(a,b)
+        assert_equal(c,a*b)
+
+    def test_single_element(self):
         a = array([4967])
         b = array([3920])
-        c = signal.fftconvolve(a,b)
-        d = a*b
-        assert_equal(c,d)
+        c = fftconvolve(a,b)
+        assert_equal(c,a*b)
 
     def test_random_data(self):
         np.random.seed(1234)
         a = np.random.rand(1233) + 1j*np.random.rand(1233)
         b = np.random.rand(1321) + 1j*np.random.rand(1321)
-        c = signal.fftconvolve(a, b, 'full')
+        c = fftconvolve(a, b, 'full')
         d = np.convolve(a, b, 'full')
-        assert np.allclose(c, d, rtol=1e-10)
+        assert_(np.allclose(c, d, rtol=1e-10))
 
 class TestMedFilt(TestCase):
     def test_basic(self):
@@ -283,18 +258,29 @@ class TestMedFilt(TestCase):
         assert_array_equal(d, e)
 
     def test_none(self):
-        """Ticket #1124. Ensure this does not segfault."""
+        # Ticket #1124. Ensure this does not segfault.
         try:
             signal.medfilt(None)
         except:
             pass
+        # Expand on this test to avoid a regression with possible contiguous
+        # numpy arrays that have odd strides. The stride value below gets
+        # us into wrong memory if used (but it does not need to be used)
+        dummy = np.arange(10, dtype=np.float64)
+        a = dummy[5:6]
+        a.strides = 16
+        assert_(signal.medfilt(a, 1) == 5.)
+
 
 class TestWiener(TestCase):
     def test_basic(self):
         g = array([[5,6,4,3],[3,5,6,2],[2,3,5,6],[1,6,9,7]],'d')
-        correct = array([[2.16374269,3.2222222222, 2.8888888889, 1.6666666667],[2.666666667, 4.33333333333, 4.44444444444, 2.8888888888],[2.222222222, 4.4444444444, 5.4444444444, 4.801066874837],[1.33333333333, 3.92735042735, 6.0712560386, 5.0404040404]])
-        h = signal.wiener(g)
-        assert_array_almost_equal(h,correct,decimal=6)
+        h = array([[2.16374269,3.2222222222, 2.8888888889, 1.6666666667],
+                   [2.666666667, 4.33333333333, 4.44444444444, 2.8888888888],
+                   [2.222222222, 4.4444444444, 5.4444444444, 4.801066874837],
+                   [1.33333333333, 3.92735042735, 6.0712560386, 5.0404040404]])
+        assert_array_almost_equal(signal.wiener(g), h , decimal=6)
+
 
 class TestCSpline1DEval(TestCase):
     def test_basic(self):
@@ -309,6 +295,7 @@ class TestCSpline1DEval(TestCase):
         # make sure interpolated values are on knot points
         assert_array_almost_equal(y2[::10], y, decimal=5)
 
+
 class TestOrderFilt(TestCase):
     def test_basic(self):
         assert_array_equal(signal.order_filter([1,2,3],[1,0,1],1),
@@ -417,15 +404,15 @@ class _TestLinearFilter(TestCase):
                 assert_array_almost_equal(y[i, j], lfilter(b, a, x[i, j]))
 
     def test_empty_zi(self):
-        """Regression test for #880: empty array for zi crashes."""
+        # Regression test for #880: empty array for zi crashes.
         a = np.ones(1).astype(self.dt)
         b = np.ones(1).astype(self.dt)
         x = np.arange(5).astype(self.dt)
         zi = np.ones(0).astype(self.dt)
         y, zf = lfilter(b, a, x, zi=zi)
         assert_array_almost_equal(y, x)
-        self.failUnless(zf.dtype == self.dt)
-        self.failUnless(zf.size == 0)
+        self.assertTrue(zf.dtype == self.dt)
+        self.assertTrue(zf.size == 0)
 
 class TestLinearFilterFloat32(_TestLinearFilter):
     dt = np.float32
@@ -448,8 +435,21 @@ class TestLinearFilterComplexxxiExtended28(_TestLinearFilter):
 class TestLinearFilterDecimal(_TestLinearFilter):
     dt = np.dtype(Decimal)
 
+class TestLinearFilterObject(_TestLinearFilter):
+    dt = np.object_
+
+
+def test_lfilter_bad_object():
+    # lfilter: object arrays with non-numeric objects raise TypeError.
+    # Regression test for ticket #1452.
+    assert_raises(TypeError, lfilter, [1.0], [1.0], [1.0, None, 2.0])
+    assert_raises(TypeError, lfilter, [1.0], [None], [1.0, 2.0, 3.0])
+    assert_raises(TypeError, lfilter, [None], [1.0], [1.0, 2.0, 3.0])
+
+
 class _TestCorrelateReal(TestCase):
     dt = None
+
     def _setup_rank1(self):
         # a.size should be greated than b.size for the tests
         a = np.linspace(0, 3, 4).astype(self.dt)
@@ -460,51 +460,27 @@ class _TestCorrelateReal(TestCase):
 
     def test_rank1_valid(self):
         a, b, y_r = self._setup_rank1()
-        y = correlate(a, b, 'valid', old_behavior=False)
+        y = correlate(a, b, 'valid')
         assert_array_almost_equal(y, y_r[1:4])
-        self.failUnless(y.dtype == self.dt)
+        self.assertTrue(y.dtype == self.dt)
 
     def test_rank1_same(self):
         a, b, y_r = self._setup_rank1()
-        y = correlate(a, b, 'same', old_behavior=False)
+        y = correlate(a, b, 'same')
         assert_array_almost_equal(y, y_r[:-1])
-        self.failUnless(y.dtype == self.dt)
+        self.assertTrue(y.dtype == self.dt)
 
     def test_rank1_full(self):
         a, b, y_r = self._setup_rank1()
-        y = correlate(a, b, 'full', old_behavior=False)
+        y = correlate(a, b, 'full')
         assert_array_almost_equal(y, y_r)
-        self.failUnless(y.dtype == self.dt)
-
-    @dec.deprecated()
-    def test_rank1_valid_old(self):
-        # This test assume a.size > b.size
-        a, b, y_r = self._setup_rank1()
-        y = correlate(b, a, 'valid')
-        assert_array_almost_equal(y, y_r[1:4])
-        self.failUnless(y.dtype == self.dt)
-
-    @dec.deprecated()
-    def test_rank1_same_old(self):
-        # This test assume a.size > b.size
-        a, b, y_r = self._setup_rank1()
-        y = correlate(b, a, 'same')
-        assert_array_almost_equal(y, y_r[:-1])
-        self.failUnless(y.dtype == self.dt)
-
-    @dec.deprecated()
-    def test_rank1_full_old(self):
-        # This test assume a.size > b.size
-        a, b, y_r = self._setup_rank1()
-        y = correlate(b, a, 'full')
-        assert_array_almost_equal(y, y_r)
-        self.failUnless(y.dtype == self.dt)
+        self.assertTrue(y.dtype == self.dt)
 
     def _setup_rank3(self):
         a = np.linspace(0, 39, 40).reshape((2, 4, 5), order='F').astype(self.dt)
         b = np.linspace(0, 23, 24).reshape((2, 3, 4), order='F').astype(self.dt)
 
-        y_r = array([[[    0.,   184.,   504.,   912.,  1360.,   888.,   472.,   160.,],
+        y_r = array([[[0.,  184.,  504.,  912., 1360.,  888.,  472.,   160.,],
             [   46.,   432.,  1062.,  1840.,  2672.,  1698.,   864.,   266.,],
             [  134.,   736.,  1662.,  2768.,  3920.,  2418.,  1168.,   314.,],
             [  260.,   952.,  1932.,  3056.,  4208.,  2580.,  1240.,   332.,] ,
@@ -530,80 +506,76 @@ class _TestCorrelateReal(TestCase):
 
     def test_rank3_valid(self):
         a, b, y_r = self._setup_rank3()
-        y = correlate(a, b, "valid", old_behavior=False)
+        y = correlate(a, b, "valid")
         assert_array_almost_equal(y, y_r[1:2,2:4,3:5])
-        self.failUnless(y.dtype == self.dt)
+        self.assertTrue(y.dtype == self.dt)
 
     def test_rank3_same(self):
         a, b, y_r = self._setup_rank3()
-        y = correlate(a, b, "same", old_behavior=False)
+        y = correlate(a, b, "same")
         assert_array_almost_equal(y, y_r[0:-1,1:-1,1:-2])
-        self.failUnless(y.dtype == self.dt)
+        self.assertTrue(y.dtype == self.dt)
 
     def test_rank3_all(self):
         a, b, y_r = self._setup_rank3()
-        y = correlate(a, b, old_behavior=False)
+        y = correlate(a, b)
         assert_array_almost_equal(y, y_r)
-        self.failUnless(y.dtype == self.dt)
+        self.assertTrue(y.dtype == self.dt)
 
-    @dec.deprecated()
-    def test_rank3_valid_old(self):
-        a, b, y_r = self._setup_rank3()
-        y = correlate(b, a, "valid")
-        assert_array_almost_equal(y, y_r[1:2,2:4,3:5])
-        self.failUnless(y.dtype == self.dt)
 
-    @dec.deprecated()
-    def test_rank3_same_old(self):
-        a, b, y_r = self._setup_rank3()
-        y = correlate(b, a, "same")
-        assert_array_almost_equal(y, y_r[0:-1,1:-1,1:-2])
-        self.failUnless(y.dtype == self.dt)
+def _get_testcorrelate_class(datatype, base):
+    class TestCorrelateX(base):
+        dt = datatype
+    TestCorrelateX.__name__ = "TestCorrelate%s" % datatype.__name__.title()
+    return TestCorrelateX
 
-    @dec.deprecated()
-    def test_rank3_all_old(self):
-        a, b, y_r = self._setup_rank3()
-        y = correlate(b, a)
-        assert_array_almost_equal(y, y_r)
-        self.failUnless(y.dtype == self.dt)
 
-for i in [np.ubyte, np.byte, np.ushort, np.short, np.uint, np.int,
+for datatype in [np.ubyte, np.byte, np.ushort, np.short, np.uint, np.int,
         np.ulonglong, np.ulonglong, np.float32, np.float64, np.longdouble,
         Decimal]:
-    name = "TestCorrelate%s" % i.__name__.title()
-    globals()[name] = types.ClassType(name, (_TestCorrelateReal,), {"dt": i})
+    cls = _get_testcorrelate_class(datatype, _TestCorrelateReal)
+    globals()[cls.__name__] = cls
+
 
 class _TestCorrelateComplex(TestCase):
+    # The numpy data type to use.
     dt = None
+
+    # The decimal precision to be used for comparing results.
+    # This value will be passed as the 'decimal' keyword argument of
+    # assert_array_almost_equal().
+    decimal = None
+
     def _setup_rank1(self, mode):
+        np.random.seed(9)
         a = np.random.randn(10).astype(self.dt)
         a += 1j * np.random.randn(10).astype(self.dt)
         b = np.random.randn(8).astype(self.dt)
         b += 1j * np.random.randn(8).astype(self.dt)
 
-        y_r = (correlate(a.real, b.real, mode=mode, old_behavior=False) +
-               correlate(a.imag, b.imag, mode=mode, old_behavior=False)).astype(self.dt)
-        y_r += 1j * (-correlate(a.real, b.imag, mode=mode, old_behavior=False) +
-                correlate(a.imag, b.real, mode=mode, old_behavior=False))
+        y_r = (correlate(a.real, b.real, mode=mode) +
+               correlate(a.imag, b.imag, mode=mode)).astype(self.dt)
+        y_r += 1j * (-correlate(a.real, b.imag, mode=mode) +
+                correlate(a.imag, b.real, mode=mode))
         return a, b, y_r
 
     def test_rank1_valid(self):
         a, b, y_r = self._setup_rank1('valid')
-        y = correlate(a, b, 'valid', old_behavior=False)
-        assert_array_almost_equal(y, y_r)
-        self.failUnless(y.dtype == self.dt)
+        y = correlate(a, b, 'valid')
+        assert_array_almost_equal(y, y_r, decimal=self.decimal)
+        self.assertTrue(y.dtype == self.dt)
 
     def test_rank1_same(self):
         a, b, y_r = self._setup_rank1('same')
-        y = correlate(a, b, 'same', old_behavior=False)
-        assert_array_almost_equal(y, y_r)
-        self.failUnless(y.dtype == self.dt)
+        y = correlate(a, b, 'same')
+        assert_array_almost_equal(y, y_r, decimal=self.decimal)
+        self.assertTrue(y.dtype == self.dt)
 
     def test_rank1_full(self):
         a, b, y_r = self._setup_rank1('full')
-        y = correlate(a, b, 'full', old_behavior=False)
-        assert_array_almost_equal(y, y_r)
-        self.failUnless(y.dtype == self.dt)
+        y = correlate(a, b, 'full')
+        assert_array_almost_equal(y, y_r, decimal=self.decimal)
+        self.assertTrue(y.dtype == self.dt)
 
     def test_rank3(self):
         a = np.random.randn(10, 8, 6).astype(self.dt)
@@ -611,68 +583,121 @@ class _TestCorrelateComplex(TestCase):
         b = np.random.randn(8, 6, 4).astype(self.dt)
         b += 1j * np.random.randn(8, 6, 4).astype(self.dt)
 
-        y_r = (correlate(a.real, b.real, old_behavior=False)
-                + correlate(a.imag, b.imag, old_behavior=False)).astype(self.dt)
-        y_r += 1j * (-correlate(a.real, b.imag, old_behavior=False) +
-                correlate(a.imag, b.real, old_behavior=False))
+        y_r = (correlate(a.real, b.real)
+                + correlate(a.imag, b.imag)).astype(self.dt)
+        y_r += 1j * (-correlate(a.real, b.imag) + correlate(a.imag, b.real))
 
-        y = correlate(a, b, 'full', old_behavior=False)
-        assert_array_almost_equal(y, y_r, decimal=4)
-        self.failUnless(y.dtype == self.dt)
+        y = correlate(a, b, 'full')
+        assert_array_almost_equal(y, y_r, decimal=self.decimal-1)
+        self.assertTrue(y.dtype == self.dt)
 
-    @dec.deprecated()
-    def test_rank1_valid_old(self):
-        a, b, y_r = self._setup_rank1('valid')
-        y = correlate(b, a.conj(), 'valid')
-        assert_array_almost_equal(y, y_r)
-        self.failUnless(y.dtype == self.dt)
 
-    @dec.deprecated()
-    def test_rank1_same_old(self):
-        a, b, y_r = self._setup_rank1('same')
-        y = correlate(b, a.conj(), 'same')
-        assert_array_almost_equal(y, y_r)
-        self.failUnless(y.dtype == self.dt)
+class TestCorrelate2d(TestCase):
+    def test_consistency_correlate_funcs(self):
+        # Compare np.correlate, signal.correlate, signal.correlate2d
+        a = np.arange(5)
+        b = np.array([3.2, 1.4, 3])
+        for mode in ['full', 'valid', 'same']:
+            assert_almost_equal(np.correlate(a, b, mode=mode),
+                                signal.correlate(a, b, mode=mode))
+            assert_almost_equal(np.squeeze(signal.correlate2d([a], [b],
+                                                              mode=mode)),
+                                signal.correlate(a, b, mode=mode))
 
-    @dec.deprecated()
-    def test_rank1_full_old(self):
-        a, b, y_r = self._setup_rank1('full')
-        y = correlate(b, a.conj(), 'full')
-        assert_array_almost_equal(y, y_r)
-        self.failUnless(y.dtype == self.dt)
 
-    @dec.deprecated()
-    def test_rank3_old(self):
-        a = np.random.randn(10, 8, 6).astype(self.dt)
-        a += 1j * np.random.randn(10, 8, 6).astype(self.dt)
-        b = np.random.randn(8, 6, 4).astype(self.dt)
-        b += 1j * np.random.randn(8, 6, 4).astype(self.dt)
+# Create three classes, one for each complex data type. The actual class
+# name will be TestCorrelateComplex###, where ### is the number of bits.
+for datatype in [np.csingle, np.cdouble, np.clongdouble]:
+    cls = _get_testcorrelate_class(datatype, _TestCorrelateComplex)
+    cls.decimal = int(2 * np.finfo(datatype).precision / 3)
+    globals()[cls.__name__] = cls
 
-        y_r = (correlate(a.real, b.real, old_behavior=False)
-                + correlate(a.imag, b.imag, old_behavior=False)).astype(self.dt)
-        y_r += 1j * (-correlate(a.real, b.imag, old_behavior=False) +
-                correlate(a.imag, b.real, old_behavior=False))
 
-        y = correlate(b, a.conj(), 'full')
-        assert_array_almost_equal(y, y_r, decimal=4)
-        self.failUnless(y.dtype == self.dt)
+class TestLFilterZI(TestCase):
 
-for i in [np.csingle, np.cdouble, np.clongdouble]:
-    name = "TestCorrelate%s" % i.__name__.title()
-    globals()[name] = types.ClassType(name, (_TestCorrelateComplex,), {"dt": i})
-
-class TestFiltFilt:
     def test_basic(self):
-        out = signal.filtfilt([1,2,3], [1,2,3], np.arange(12))
+        a = np.array([1.0, -1.0, 0.5])
+        b = np.array([1.0,  0.0, 2.0])
+        zi_expected = np.array([5.0, -1.0])
+        zi = lfilter_zi(b, a)
+        assert_array_almost_equal(zi, zi_expected)
+
+
+class TestFiltFilt(TestCase):
+
+    def test_basic(self):
+        out = signal.filtfilt([1, 2, 3], [1, 2, 3], np.arange(12))
         assert_equal(out, arange(12))
 
+    def test_sine(self):
+        rate = 2000
+        t = np.linspace(0, 1.0, rate + 1)
+        # A signal with low frequency and a high frequency.
+        xlow = np.sin(5 * 2 * np.pi * t)
+        xhigh = np.sin(250 * 2 * np.pi * t)
+        x = xlow + xhigh
+
+        b, a = butter(8, 0.125)
+        z, p, k = tf2zpk(b, a)
+        # r is the magnitude of the largest pole.
+        r = np.abs(p).max()
+        eps = 1e-5
+        # n estimates the number of steps for the
+        # transient to decay by a factor of eps.
+        n = int(np.ceil(np.log(eps) / np.log(r)))
+
+        # High order lowpass filter...
+        y = filtfilt(b, a, x, padlen=n)
+        # Result should be just xlow.
+        err = np.abs(y - xlow).max()
+        assert_(err < 1e-4)
+
+        # A 2D case.
+        x2d = np.vstack([xlow, xlow + xhigh])
+        y2d = filtfilt(b, a, x2d, padlen=n, axis=1)
+        assert_equal(y2d.shape, x2d.shape)
+        err = np.abs(y2d - xlow).max()
+        assert_(err < 1e-4)
+
+        # Use the previous result to check the use of the axis keyword.
+        # (Regression test for ticket #1620)
+        y2dt = filtfilt(b, a, x2d.T, padlen=n, axis=0)
+        assert_equal(y2d, y2dt.T)
+
+    def test_axis(self):
+        # Test the 'axis' keyword on a 3D array.
+        x = np.arange(10.0 * 11.0 * 12.0).reshape(10, 11, 12)
+        b, a = butter(3, 0.125)
+        y0 = filtfilt(b, a, x, padlen=0, axis=0)
+        y1 = filtfilt(b, a, np.swapaxes(x, 0, 1), padlen=0, axis=1)
+        assert_array_equal(y0, np.swapaxes(y1, 0, 1))
+        y2 = filtfilt(b, a, np.swapaxes(x, 0, 2), padlen=0, axis=2)
+        assert_array_equal(y0, np.swapaxes(y2, 0, 2))
+
+
 class TestDecimate:
+
     def test_basic(self):
         x = np.arange(6)
         assert_array_equal(signal.decimate(x, 2, n=1).round(), x[::2])
 
+    def test_shape(self):
+        # Regression test for ticket #1480.
+        z = np.zeros((10, 10))
+        d0 = signal.decimate(z, 2, axis=0)
+        assert_equal(d0.shape, (5, 10))
+        d1 = signal.decimate(z, 2, axis=1)
+        assert_equal(d1.shape, (10, 5))
+
+
+class TestHilbert(object):
+
+    def test_bad_args(self):
+        x = np.array([1.0+0.0j])
+        assert_raises(ValueError, hilbert, x)
+        x = np.arange(8.0)
+        assert_raises(ValueError, hilbert, x, N=0)
 
-class TestHilbert:
     def test_hilbert_theoretical(self):
         #test cases by Ariel Rokem
         decimal = 14
@@ -725,8 +750,7 @@ class TestHilbert:
         yield assert_equal, hilbert(a.T, N=20, axis=0).shape, [20,3]
         #the next test is just a regression test,
         #no idea whether numbers make sense
-        a0hilb = np.array(
-                          [  0.000000000000000e+00-1.72015830311905j ,
+        a0hilb = np.array([  0.000000000000000e+00-1.72015830311905j ,
                              1.000000000000000e+00-2.047794505137069j,
                              1.999999999999999e+00-2.244055555687583j,
                              3.000000000000000e+00-1.262750302935009j,
@@ -749,5 +773,23 @@ class TestHilbert:
         yield assert_almost_equal, aan[0], a0hilb, 14, 'N regression'
 
 
+class TestHilbert2(object):
+
+    def test_bad_args(self):
+        # x must be real.
+        x = np.array([[1.0 + 0.0j]])
+        assert_raises(ValueError, hilbert2, x)
+
+        # x must be rank 2.
+        x = np.arange(24).reshape(2, 3, 4)
+        assert_raises(ValueError, hilbert2, x)
+
+        # Bad value for N.
+        x = np.arange(16).reshape(4, 4)
+        assert_raises(ValueError, hilbert2, x, N=0)
+        assert_raises(ValueError, hilbert2, x, N=(2,0))
+        assert_raises(ValueError, hilbert2, x, N=(2,))
+
+
 if __name__ == "__main__":
     run_module_suite()
